#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_system_central_management
short_description: Configure central management.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure system feature and central_management category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    system_central_management:
        description:
            - Configure central management.
        default: null
        suboptions:
            allow-monitor:
                description:
                    - Enable/disable allowing the central management server to remotely monitor this FortiGate
                choices:
                    - enable
                    - disable
            allow-push-configuration:
                description:
                    - Enable/disable allowing the central management server to push configuration changes to this FortiGate.
                choices:
                    - enable
                    - disable
            allow-push-firmware:
                description:
                    - Enable/disable allowing the central management server to push firmware updates to this FortiGate.
                choices:
                    - enable
                    - disable
            allow-remote-firmware-upgrade:
                description:
                    - Enable/disable remotely upgrading the firmware on this FortiGate from the central management server.
                choices:
                    - enable
                    - disable
            enc-algorithm:
                description:
                    - Encryption strength for communications between the FortiGate and central management.
                choices:
                    - default
                    - high
                    - low
            fmg:
                description:
                    - IP address or FQDN of the FortiManager.
            fmg-source-ip:
                description:
                    - IPv4 source address that this FortiGate uses when communicating with FortiManager.
            fmg-source-ip6:
                description:
                    - IPv6 source address that this FortiGate uses when communicating with FortiManager.
            include-default-servers:
                description:
                    - Enable/disable inclusion of public FortiGuard servers in the override server list.
                choices:
                    - enable
                    - disable
            mode:
                description:
                    - Central management mode.
                choices:
                    - normal
                    - backup
            schedule-config-restore:
                description:
                    - Enable/disable allowing the central management server to restore the configuration of this FortiGate.
                choices:
                    - enable
                    - disable
            schedule-script-restore:
                description:
                    - Enable/disable allowing the central management server to restore the scripts stored on this FortiGate.
                choices:
                    - enable
                    - disable
            serial-number:
                description:
                    - Serial number.
            server-list:
                description:
                    - Additional severs that the FortiGate can use for updates (for AV, IPS, updates) and ratings (for web filter and antispam ratings)
                       servers.
                suboptions:
                    addr-type:
                        description:
                            - Indicate whether the FortiGate communicates with the override server using an IPv4 address, an IPv6 address or a FQDN.
                        choices:
                            - ipv4
                            - ipv6
                            - fqdn
                    fqdn:
                        description:
                            - FQDN address of override server.
                    id:
                        description:
                            - ID.
                        required: true
                    server-address:
                        description:
                            - IPv4 address of override server.
                    server-address6:
                        description:
                            - IPv6 address of override server.
                    server-type:
                        description:
                            - FortiGuard service type.
                        choices:
                            - update
                            - rating
            type:
                description:
                    - Central management type.
                choices:
                    - fortimanager
                    - fortiguard
                    - none
            vdom:
                description:
                    - Virtual domain (VDOM) name to use when communicating with FortiManager. Source system.vdom.name.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure central management.
    fortios_system_central_management:
      host:  "{{  host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{  vdom }}"
      system_central_management:
        allow-monitor: "enable"
        allow-push-configuration: "enable"
        allow-push-firmware: "enable"
        allow-remote-firmware-upgrade: "enable"
        enc-algorithm: "default"
        fmg: "<your_own_value>"
        fmg-source-ip: "<your_own_value>"
        fmg-source-ip6: "<your_own_value>"
        include-default-servers: "enable"
        mode: "normal"
        schedule-config-restore: "enable"
        schedule-script-restore: "enable"
        serial-number: "<your_own_value>"
        server-list:
         -
            addr-type: "ipv4"
            fqdn: "<your_own_value>"
            id:  "19"
            server-address: "<your_own_value>"
            server-address6: "<your_own_value>"
            server-type: "update"
        type: "fortimanager"
        vdom: "<your_own_value> (source system.vdom.name)"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_system_central_management_data(json):
    option_list = ['allow-monitor', 'allow-push-configuration', 'allow-push-firmware',
                   'allow-remote-firmware-upgrade', 'enc-algorithm', 'fmg',
                   'fmg-source-ip', 'fmg-source-ip6', 'include-default-servers',
                   'mode', 'schedule-config-restore', 'schedule-script-restore',
                   'serial-number', 'server-list', 'type',
                   'vdom']
    dictionary = {}

    for attribute in option_list:
        if attribute in json:
            dictionary[attribute] = json[attribute]

    return dictionary


def system_central_management(data, fos):
    vdom = data['vdom']
    system_central_management_data = data['system_central_management']
    filtered_data = filter_system_central_management_data(
        system_central_management_data)
    return fos.set('system',
                   'central-management',
                   data=filtered_data,
                   vdom=vdom)


def fortios_system(data, fos):
    login(data)

    methodlist = ['system_central_management']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "system_central_management": {
            "required": False, "type": "dict",
            "options": {
                "allow-monitor": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "allow-push-configuration": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "allow-push-firmware": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "allow-remote-firmware-upgrade": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "enc-algorithm": {"required": False, "type": "str",
                                  "choices": ["default", "high", "low"]},
                "fmg": {"required": False, "type": "str"},
                "fmg-source-ip": {"required": False, "type": "str"},
                "fmg-source-ip6": {"required": False, "type": "str"},
                "include-default-servers": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "mode": {"required": False, "type": "str",
                         "choices": ["normal", "backup"]},
                "schedule-config-restore": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "schedule-script-restore": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "serial-number": {"required": False, "type": "str"},
                "server-list": {"required": False, "type": "list",
                                "options": {
                                    "addr-type": {"required": False, "type": "str",
                                                  "choices": ["ipv4", "ipv6", "fqdn"]},
                                    "fqdn": {"required": False, "type": "str"},
                                    "id": {"required": True, "type": "int"},
                                    "server-address": {"required": False, "type": "str"},
                                    "server-address6": {"required": False, "type": "str"},
                                    "server-type": {"required": False, "type": "str",
                                                    "choices": ["update", "rating"]}
                                }},
                "type": {"required": False, "type": "str",
                         "choices": ["fortimanager", "fortiguard", "none"]},
                "vdom": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_system(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_policy
short_description: Configure IPv4 policies.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and policy category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    firewall_policy:
        description:
            - Configure IPv4 policies.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            action:
                description:
                    - Policy action (allow/deny/ipsec).
                choices:
                    - accept
                    - deny
                    - ipsec
            app-category:
                description:
                    - Application category ID list.
                suboptions:
                    id:
                        description:
                            - Category IDs.
                        required: true
            app-group:
                description:
                    - Application group names.
                suboptions:
                    name:
                        description:
                            - Application group names. Source application.group.name.
                        required: true
            application:
                description:
                    - Application ID list.
                suboptions:
                    id:
                        description:
                            - Application IDs.
                        required: true
            application-list:
                description:
                    - Name of an existing Application list. Source application.list.name.
            auth-cert:
                description:
                    - HTTPS server certificate for policy authentication. Source vpn.certificate.local.name.
            auth-path:
                description:
                    - Enable/disable authentication-based routing.
                choices:
                    - enable
                    - disable
            auth-redirect-addr:
                description:
                    - HTTP-to-HTTPS redirect address for firewall authentication.
            av-profile:
                description:
                    - Name of an existing Antivirus profile. Source antivirus.profile.name.
            block-notification:
                description:
                    - Enable/disable block notification.
                choices:
                    - enable
                    - disable
            captive-portal-exempt:
                description:
                    - Enable to exempt some users from the captive portal.
                choices:
                    - enable
                    - disable
            capture-packet:
                description:
                    - Enable/disable capture packets.
                choices:
                    - enable
                    - disable
            comments:
                description:
                    - Comment.
            custom-log-fields:
                description:
                    - Custom fields to append to log messages for this policy.
                suboptions:
                    field-id:
                        description:
                            - Custom log field. Source log.custom-field.id.
                        required: true
            delay-tcp-npu-session:
                description:
                    - Enable TCP NPU session delay to guarantee packet order of 3-way handshake.
                choices:
                    - enable
                    - disable
            devices:
                description:
                    - Names of devices or device groups that can be matched by the policy.
                suboptions:
                    name:
                        description:
                            - Device or group name. Source user.device.alias user.device-group.name user.device-category.name.
                        required: true
            diffserv-forward:
                description:
                    - Enable to change packet's DiffServ values to the specified diffservcode-forward value.
                choices:
                    - enable
                    - disable
            diffserv-reverse:
                description:
                    - Enable to change packet's reverse (reply) DiffServ values to the specified diffservcode-rev value.
                choices:
                    - enable
                    - disable
            diffservcode-forward:
                description:
                    - Change packet's DiffServ to this value.
            diffservcode-rev:
                description:
                    - Change packet's reverse (reply) DiffServ to this value.
            disclaimer:
                description:
                    - Enable/disable user authentication disclaimer.
                choices:
                    - enable
                    - disable
            dlp-sensor:
                description:
                    - Name of an existing DLP sensor. Source dlp.sensor.name.
            dnsfilter-profile:
                description:
                    - Name of an existing DNS filter profile. Source dnsfilter.profile.name.
            dscp-match:
                description:
                    - Enable DSCP check.
                choices:
                    - enable
                    - disable
            dscp-negate:
                description:
                    - Enable negated DSCP match.
                choices:
                    - enable
                    - disable
            dscp-value:
                description:
                    - DSCP value.
            dsri:
                description:
                    - Enable DSRI to ignore HTTP server responses.
                choices:
                    - enable
                    - disable
            dstaddr:
                description:
                    - Destination address and address group names.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address.name firewall.addrgrp.name firewall.vip.name firewall.vipgrp.name.
                        required: true
            dstaddr-negate:
                description:
                    - When enabled dstaddr specifies what the destination address must NOT be.
                choices:
                    - enable
                    - disable
            dstintf:
                description:
                    - Outgoing (egress) interface.
                suboptions:
                    name:
                        description:
                            - Interface name. Source system.interface.name system.zone.name.
                        required: true
            firewall-session-dirty:
                description:
                    - How to handle sessions if the configuration of this firewall policy changes.
                choices:
                    - check-all
                    - check-new
            fixedport:
                description:
                    - Enable to prevent source NAT from changing a session's source port.
                choices:
                    - enable
                    - disable
            fsso:
                description:
                    - Enable/disable Fortinet Single Sign-On.
                choices:
                    - enable
                    - disable
            fsso-agent-for-ntlm:
                description:
                    - FSSO agent to use for NTLM authentication. Source user.fsso.name.
            global-label:
                description:
                    - Label for the policy that appears when the GUI is in Global View mode.
            groups:
                description:
                    - Names of user groups that can authenticate with this policy.
                suboptions:
                    name:
                        description:
                            - Group name. Source user.group.name.
                        required: true
            icap-profile:
                description:
                    - Name of an existing ICAP profile. Source icap.profile.name.
            identity-based-route:
                description:
                    - Name of identity-based routing rule. Source firewall.identity-based-route.name.
            inbound:
                description:
                    - "Policy-based IPsec VPN: only traffic from the remote network can initiate a VPN."
                choices:
                    - enable
                    - disable
            internet-service:
                description:
                    - Enable/disable use of Internet Services for this policy. If enabled, destination address and service are not used.
                choices:
                    - enable
                    - disable
            internet-service-custom:
                description:
                    - Custom Internet Service name.
                suboptions:
                    name:
                        description:
                            - Custom Internet Service name. Source firewall.internet-service-custom.name.
                        required: true
            internet-service-id:
                description:
                    - Internet Service ID.
                suboptions:
                    id:
                        description:
                            - Internet Service ID. Source firewall.internet-service.id.
                        required: true
            internet-service-negate:
                description:
                    - When enabled internet-service specifies what the service must NOT be.
                choices:
                    - enable
                    - disable
            internet-service-src:
                description:
                    - Enable/disable use of Internet Services in source for this policy. If enabled, source address is not used.
                choices:
                    - enable
                    - disable
            internet-service-src-custom:
                description:
                    - Custom Internet Service source name.
                suboptions:
                    name:
                        description:
                            - Custom Internet Service name. Source firewall.internet-service-custom.name.
                        required: true
            internet-service-src-id:
                description:
                    - Internet Service source ID.
                suboptions:
                    id:
                        description:
                            - Internet Service ID. Source firewall.internet-service.id.
                        required: true
            internet-service-src-negate:
                description:
                    - When enabled internet-service-src specifies what the service must NOT be.
                choices:
                    - enable
                    - disable
            ippool:
                description:
                    - Enable to use IP Pools for source NAT.
                choices:
                    - enable
                    - disable
            ips-sensor:
                description:
                    - Name of an existing IPS sensor. Source ips.sensor.name.
            label:
                description:
                    - Label for the policy that appears when the GUI is in Section View mode.
            learning-mode:
                description:
                    - Enable to allow everything, but log all of the meaningful data for security information gathering. A learning report will be generated.
                choices:
                    - enable
                    - disable
            logtraffic:
                description:
                    - Enable or disable logging. Log all sessions or security profile sessions.
                choices:
                    - all
                    - utm
                    - disable
            logtraffic-start:
                description:
                    - Record logs when a session starts and ends.
                choices:
                    - enable
                    - disable
            match-vip:
                description:
                    - Enable to match packets that have had their destination addresses changed by a VIP.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - Policy name.
            nat:
                description:
                    - Enable/disable source NAT.
                choices:
                    - enable
                    - disable
            natinbound:
                description:
                    - "Policy-based IPsec VPN: apply destination NAT to inbound traffic."
                choices:
                    - enable
                    - disable
            natip:
                description:
                    - "Policy-based IPsec VPN: source NAT IP address for outgoing traffic."
            natoutbound:
                description:
                    - "Policy-based IPsec VPN: apply source NAT to outbound traffic."
                choices:
                    - enable
                    - disable
            ntlm:
                description:
                    - Enable/disable NTLM authentication.
                choices:
                    - enable
                    - disable
            ntlm-enabled-browsers:
                description:
                    - HTTP-User-Agent value of supported browsers.
                suboptions:
                    user-agent-string:
                        description:
                            - User agent string.
                        required: true
            ntlm-guest:
                description:
                    - Enable/disable NTLM guest user access.
                choices:
                    - enable
                    - disable
            outbound:
                description:
                    - "Policy-based IPsec VPN: only traffic from the internal network can initiate a VPN."
                choices:
                    - enable
                    - disable
            per-ip-shaper:
                description:
                    - Per-IP traffic shaper. Source firewall.shaper.per-ip-shaper.name.
            permit-any-host:
                description:
                    - Accept UDP packets from any host.
                choices:
                    - enable
                    - disable
            permit-stun-host:
                description:
                    - Accept UDP packets from any Session Traversal Utilities for NAT (STUN) host.
                choices:
                    - enable
                    - disable
            policyid:
                description:
                    - Policy ID.
                required: true
            poolname:
                description:
                    - IP Pool names.
                suboptions:
                    name:
                        description:
                            - IP pool name. Source firewall.ippool.name.
                        required: true
            profile-group:
                description:
                    - Name of profile group. Source firewall.profile-group.name.
            profile-protocol-options:
                description:
                    - Name of an existing Protocol options profile. Source firewall.profile-protocol-options.name.
            profile-type:
                description:
                    - Determine whether the firewall policy allows security profile groups or single profiles only.
                choices:
                    - single
                    - group
            radius-mac-auth-bypass:
                description:
                    - Enable MAC authentication bypass. The bypassed MAC address must be received from RADIUS server.
                choices:
                    - enable
                    - disable
            redirect-url:
                description:
                    - URL users are directed to after seeing and accepting the disclaimer or authenticating.
            replacemsg-override-group:
                description:
                    - Override the default replacement message group for this policy. Source system.replacemsg-group.name.
            rsso:
                description:
                    - Enable/disable RADIUS single sign-on (RSSO).
                choices:
                    - enable
                    - disable
            rtp-addr:
                description:
                    - Address names if this is an RTP NAT policy.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            rtp-nat:
                description:
                    - Enable Real Time Protocol (RTP) NAT.
                choices:
                    - disable
                    - enable
            scan-botnet-connections:
                description:
                    - Block or monitor connections to Botnet servers or disable Botnet scanning.
                choices:
                    - disable
                    - block
                    - monitor
            schedule:
                description:
                    - Schedule name. Source firewall.schedule.onetime.name firewall.schedule.recurring.name firewall.schedule.group.name.
            schedule-timeout:
                description:
                    - Enable to force current sessions to end when the schedule object times out. Disable allows them to end from inactivity.
                choices:
                    - enable
                    - disable
            send-deny-packet:
                description:
                    - Enable to send a reply when a session is denied or blocked by a firewall policy.
                choices:
                    - disable
                    - enable
            service:
                description:
                    - Service and service group names.
                suboptions:
                    name:
                        description:
                            - Service and service group names. Source firewall.service.custom.name firewall.service.group.name.
                        required: true
            service-negate:
                description:
                    - When enabled service specifies what the service must NOT be.
                choices:
                    - enable
                    - disable
            session-ttl:
                description:
                    - TTL in seconds for sessions accepted by this policy (0 means use the system default session TTL).
            spamfilter-profile:
                description:
                    - Name of an existing Spam filter profile. Source spamfilter.profile.name.
            srcaddr:
                description:
                    - Source address and address group names.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            srcaddr-negate:
                description:
                    - When enabled srcaddr specifies what the source address must NOT be.
                choices:
                    - enable
                    - disable
            srcintf:
                description:
                    - Incoming (ingress) interface.
                suboptions:
                    name:
                        description:
                            - Interface name. Source system.interface.name system.zone.name.
                        required: true
            ssh-filter-profile:
                description:
                    - Name of an existing SSH filter profile. Source ssh-filter.profile.name.
            ssl-mirror:
                description:
                    - Enable to copy decrypted SSL traffic to a FortiGate interface (called SSL mirroring).
                choices:
                    - enable
                    - disable
            ssl-mirror-intf:
                description:
                    - SSL mirror interface name.
                suboptions:
                    name:
                        description:
                            - Mirror Interface name. Source system.interface.name system.zone.name.
                        required: true
            ssl-ssh-profile:
                description:
                    - Name of an existing SSL SSH profile. Source firewall.ssl-ssh-profile.name.
            status:
                description:
                    - Enable or disable this policy.
                choices:
                    - enable
                    - disable
            tcp-mss-receiver:
                description:
                    - Receiver TCP maximum segment size (MSS).
            tcp-mss-sender:
                description:
                    - Sender TCP maximum segment size (MSS).
            tcp-session-without-syn:
                description:
                    - Enable/disable creation of TCP session without SYN flag.
                choices:
                    - all
                    - data-only
                    - disable
            timeout-send-rst:
                description:
                    - Enable/disable sending RST packets when TCP sessions expire.
                choices:
                    - enable
                    - disable
            traffic-shaper:
                description:
                    - Traffic shaper. Source firewall.shaper.traffic-shaper.name.
            traffic-shaper-reverse:
                description:
                    - Reverse traffic shaper. Source firewall.shaper.traffic-shaper.name.
            url-category:
                description:
                    - URL category ID list.
                suboptions:
                    id:
                        description:
                            - URL category ID.
                        required: true
            users:
                description:
                    - Names of individual users that can authenticate with this policy.
                suboptions:
                    name:
                        description:
                            - Names of individual users that can authenticate with this policy. Source user.local.name.
                        required: true
            utm-status:
                description:
                    - Enable to add one or more security profiles (AV, IPS, etc.) to the firewall policy.
                choices:
                    - enable
                    - disable
            uuid:
                description:
                    - Universally Unique Identifier (UUID; automatically assigned but can be manually reset).
            vlan-cos-fwd:
                description:
                    - "VLAN forward direction user priority: 255 passthrough, 0 lowest, 7 highest."
            vlan-cos-rev:
                description:
                    - "VLAN reverse direction user priority: 255 passthrough, 0 lowest, 7 highest.."
            vlan-filter:
                description:
                    - Set VLAN filters.
            voip-profile:
                description:
                    - Name of an existing VoIP profile. Source voip.profile.name.
            vpntunnel:
                description:
                    - "Policy-based IPsec VPN: name of the IPsec VPN Phase 1. Source vpn.ipsec.phase1.name vpn.ipsec.manualkey.name."
            waf-profile:
                description:
                    - Name of an existing Web application firewall profile. Source waf.profile.name.
            wanopt:
                description:
                    - Enable/disable WAN optimization.
                choices:
                    - enable
                    - disable
            wanopt-detection:
                description:
                    - WAN optimization auto-detection mode.
                choices:
                    - active
                    - passive
                    - off
            wanopt-passive-opt:
                description:
                    - WAN optimization passive mode options. This option decides what IP address will be used to connect server.
                choices:
                    - default
                    - transparent
                    - non-transparent
            wanopt-peer:
                description:
                    - WAN optimization peer. Source wanopt.peer.peer-host-id.
            wanopt-profile:
                description:
                    - WAN optimization profile. Source wanopt.profile.name.
            wccp:
                description:
                    - Enable/disable forwarding traffic matching this policy to a configured WCCP server.
                choices:
                    - enable
                    - disable
            webcache:
                description:
                    - Enable/disable web cache.
                choices:
                    - enable
                    - disable
            webcache-https:
                description:
                    - Enable/disable web cache for HTTPS.
                choices:
                    - disable
                    - enable
            webfilter-profile:
                description:
                    - Name of an existing Web filter profile. Source webfilter.profile.name.
            wsso:
                description:
                    - Enable/disable WiFi Single Sign On (WSSO).
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPv4 policies.
    fortios_firewall_policy:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      firewall_policy:
        state: "present"
        action: "accept"
        app-category:
         -
            id:  "5"
        app-group:
         -
            name: "default_name_7 (source application.group.name)"
        application:
         -
            id:  "9"
        application-list: "<your_own_value> (source application.list.name)"
        auth-cert: "<your_own_value> (source vpn.certificate.local.name)"
        auth-path: "enable"
        auth-redirect-addr: "<your_own_value>"
        av-profile: "<your_own_value> (source antivirus.profile.name)"
        block-notification: "enable"
        captive-portal-exempt: "enable"
        capture-packet: "enable"
        comments: "<your_own_value>"
        custom-log-fields:
         -
            field-id: "<your_own_value> (source log.custom-field.id)"
        delay-tcp-npu-session: "enable"
        devices:
         -
            name: "default_name_23 (source user.device.alias user.device-group.name user.device-category.name)"
        diffserv-forward: "enable"
        diffserv-reverse: "enable"
        diffservcode-forward: "<your_own_value>"
        diffservcode-rev: "<your_own_value>"
        disclaimer: "enable"
        dlp-sensor: "<your_own_value> (source dlp.sensor.name)"
        dnsfilter-profile: "<your_own_value> (source dnsfilter.profile.name)"
        dscp-match: "enable"
        dscp-negate: "enable"
        dscp-value: "<your_own_value>"
        dsri: "enable"
        dstaddr:
         -
            name: "default_name_36 (source firewall.address.name firewall.addrgrp.name firewall.vip.name firewall.vipgrp.name)"
        dstaddr-negate: "enable"
        dstintf:
         -
            name: "default_name_39 (source system.interface.name system.zone.name)"
        firewall-session-dirty: "check-all"
        fixedport: "enable"
        fsso: "enable"
        fsso-agent-for-ntlm: "<your_own_value> (source user.fsso.name)"
        global-label: "<your_own_value>"
        groups:
         -
            name: "default_name_46 (source user.group.name)"
        icap-profile: "<your_own_value> (source icap.profile.name)"
        identity-based-route: "<your_own_value> (source firewall.identity-based-route.name)"
        inbound: "enable"
        internet-service: "enable"
        internet-service-custom:
         -
            name: "default_name_52 (source firewall.internet-service-custom.name)"
        internet-service-id:
         -
            id:  "54 (source firewall.internet-service.id)"
        internet-service-negate: "enable"
        internet-service-src: "enable"
        internet-service-src-custom:
         -
            name: "default_name_58 (source firewall.internet-service-custom.name)"
        internet-service-src-id:
         -
            id:  "60 (source firewall.internet-service.id)"
        internet-service-src-negate: "enable"
        ippool: "enable"
        ips-sensor: "<your_own_value> (source ips.sensor.name)"
        label: "<your_own_value>"
        learning-mode: "enable"
        logtraffic: "all"
        logtraffic-start: "enable"
        match-vip: "enable"
        name: "default_name_69"
        nat: "enable"
        natinbound: "enable"
        natip: "<your_own_value>"
        natoutbound: "enable"
        ntlm: "enable"
        ntlm-enabled-browsers:
         -
            user-agent-string: "<your_own_value>"
        ntlm-guest: "enable"
        outbound: "enable"
        per-ip-shaper: "<your_own_value> (source firewall.shaper.per-ip-shaper.name)"
        permit-any-host: "enable"
        permit-stun-host: "enable"
        policyid: "82"
        poolname:
         -
            name: "default_name_84 (source firewall.ippool.name)"
        profile-group: "<your_own_value> (source firewall.profile-group.name)"
        profile-protocol-options: "<your_own_value> (source firewall.profile-protocol-options.name)"
        profile-type: "single"
        radius-mac-auth-bypass: "enable"
        redirect-url: "<your_own_value>"
        replacemsg-override-group: "<your_own_value> (source system.replacemsg-group.name)"
        rsso: "enable"
        rtp-addr:
         -
            name: "default_name_93 (source firewall.address.name firewall.addrgrp.name)"
        rtp-nat: "disable"
        scan-botnet-connections: "disable"
        schedule: "<your_own_value> (source firewall.schedule.onetime.name firewall.schedule.recurring.name firewall.schedule.group.name)"
        schedule-timeout: "enable"
        send-deny-packet: "disable"
        service:
         -
            name: "default_name_100 (source firewall.service.custom.name firewall.service.group.name)"
        service-negate: "enable"
        session-ttl: "102"
        spamfilter-profile: "<your_own_value> (source spamfilter.profile.name)"
        srcaddr:
         -
            name: "default_name_105 (source firewall.address.name firewall.addrgrp.name)"
        srcaddr-negate: "enable"
        srcintf:
         -
            name: "default_name_108 (source system.interface.name system.zone.name)"
        ssh-filter-profile: "<your_own_value> (source ssh-filter.profile.name)"
        ssl-mirror: "enable"
        ssl-mirror-intf:
         -
            name: "default_name_112 (source system.interface.name system.zone.name)"
        ssl-ssh-profile: "<your_own_value> (source firewall.ssl-ssh-profile.name)"
        status: "enable"
        tcp-mss-receiver: "115"
        tcp-mss-sender: "116"
        tcp-session-without-syn: "all"
        timeout-send-rst: "enable"
        traffic-shaper: "<your_own_value> (source firewall.shaper.traffic-shaper.name)"
        traffic-shaper-reverse: "<your_own_value> (source firewall.shaper.traffic-shaper.name)"
        url-category:
         -
            id:  "122"
        users:
         -
            name: "default_name_124 (source user.local.name)"
        utm-status: "enable"
        uuid: "<your_own_value>"
        vlan-cos-fwd: "127"
        vlan-cos-rev: "128"
        vlan-filter: "<your_own_value>"
        voip-profile: "<your_own_value> (source voip.profile.name)"
        vpntunnel: "<your_own_value> (source vpn.ipsec.phase1.name vpn.ipsec.manualkey.name)"
        waf-profile: "<your_own_value> (source waf.profile.name)"
        wanopt: "enable"
        wanopt-detection: "active"
        wanopt-passive-opt: "default"
        wanopt-peer: "<your_own_value> (source wanopt.peer.peer-host-id)"
        wanopt-profile: "<your_own_value> (source wanopt.profile.name)"
        wccp: "enable"
        webcache: "enable"
        webcache-https: "disable"
        webfilter-profile: "<your_own_value> (source webfilter.profile.name)"
        wsso: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_policy_data(json):
    option_list = ['action', 'app-category', 'app-group',
                   'application', 'application-list', 'auth-cert',
                   'auth-path', 'auth-redirect-addr', 'av-profile',
                   'block-notification', 'captive-portal-exempt', 'capture-packet',
                   'comments', 'custom-log-fields', 'delay-tcp-npu-session',
                   'devices', 'diffserv-forward', 'diffserv-reverse',
                   'diffservcode-forward', 'diffservcode-rev', 'disclaimer',
                   'dlp-sensor', 'dnsfilter-profile', 'dscp-match',
                   'dscp-negate', 'dscp-value', 'dsri',
                   'dstaddr', 'dstaddr-negate', 'dstintf',
                   'firewall-session-dirty', 'fixedport', 'fsso',
                   'fsso-agent-for-ntlm', 'global-label', 'groups',
                   'icap-profile', 'identity-based-route', 'inbound',
                   'internet-service', 'internet-service-custom', 'internet-service-id',
                   'internet-service-negate', 'internet-service-src', 'internet-service-src-custom',
                   'internet-service-src-id', 'internet-service-src-negate', 'ippool',
                   'ips-sensor', 'label', 'learning-mode',
                   'logtraffic', 'logtraffic-start', 'match-vip',
                   'name', 'nat', 'natinbound',
                   'natip', 'natoutbound', 'ntlm',
                   'ntlm-enabled-browsers', 'ntlm-guest', 'outbound',
                   'per-ip-shaper', 'permit-any-host', 'permit-stun-host',
                   'policyid', 'poolname', 'profile-group',
                   'profile-protocol-options', 'profile-type', 'radius-mac-auth-bypass',
                   'redirect-url', 'replacemsg-override-group', 'rsso',
                   'rtp-addr', 'rtp-nat', 'scan-botnet-connections',
                   'schedule', 'schedule-timeout', 'send-deny-packet',
                   'service', 'service-negate', 'session-ttl',
                   'spamfilter-profile', 'srcaddr', 'srcaddr-negate',
                   'srcintf', 'ssh-filter-profile', 'ssl-mirror',
                   'ssl-mirror-intf', 'ssl-ssh-profile', 'status',
                   'tcp-mss-receiver', 'tcp-mss-sender', 'tcp-session-without-syn',
                   'timeout-send-rst', 'traffic-shaper', 'traffic-shaper-reverse',
                   'url-category', 'users', 'utm-status',
                   'uuid', 'vlan-cos-fwd', 'vlan-cos-rev',
                   'vlan-filter', 'voip-profile', 'vpntunnel',
                   'waf-profile', 'wanopt', 'wanopt-detection',
                   'wanopt-passive-opt', 'wanopt-peer', 'wanopt-profile',
                   'wccp', 'webcache', 'webcache-https',
                   'webfilter-profile', 'wsso']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_policy(data, fos):
    vdom = data['vdom']
    firewall_policy_data = data['firewall_policy']
    filtered_data = filter_firewall_policy_data(firewall_policy_data)
    if firewall_policy_data['state'] == "present":
        return fos.set('firewall',
                       'policy',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_policy_data['state'] == "absent":
        return fos.delete('firewall',
                          'policy',
                          mkey=filtered_data['policyid'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_policy']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "firewall_policy": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "action": {"required": False, "type": "str",
                           "choices": ["accept", "deny", "ipsec"]},
                "app-category": {"required": False, "type": "list",
                                 "options": {
                                     "id": {"required": True, "type": "int"}
                                 }},
                "app-group": {"required": False, "type": "list",
                              "options": {
                                  "name": {"required": True, "type": "str"}
                              }},
                "application": {"required": False, "type": "list",
                                "options": {
                                    "id": {"required": True, "type": "int"}
                                }},
                "application-list": {"required": False, "type": "str"},
                "auth-cert": {"required": False, "type": "str"},
                "auth-path": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "auth-redirect-addr": {"required": False, "type": "str"},
                "av-profile": {"required": False, "type": "str"},
                "block-notification": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "captive-portal-exempt": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "capture-packet": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "comments": {"required": False, "type": "str"},
                "custom-log-fields": {"required": False, "type": "list",
                                      "options": {
                                          "field-id": {"required": True, "type": "str"}
                                      }},
                "delay-tcp-npu-session": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "devices": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "diffserv-forward": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "diffserv-reverse": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "diffservcode-forward": {"required": False, "type": "str"},
                "diffservcode-rev": {"required": False, "type": "str"},
                "disclaimer": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "dlp-sensor": {"required": False, "type": "str"},
                "dnsfilter-profile": {"required": False, "type": "str"},
                "dscp-match": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "dscp-negate": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "dscp-value": {"required": False, "type": "str"},
                "dsri": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "dstaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "dstaddr-negate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "dstintf": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "firewall-session-dirty": {"required": False, "type": "str",
                                           "choices": ["check-all", "check-new"]},
                "fixedport": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "fsso": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "fsso-agent-for-ntlm": {"required": False, "type": "str"},
                "global-label": {"required": False, "type": "str"},
                "groups": {"required": False, "type": "list",
                           "options": {
                               "name": {"required": True, "type": "str"}
                           }},
                "icap-profile": {"required": False, "type": "str"},
                "identity-based-route": {"required": False, "type": "str"},
                "inbound": {"required": False, "type": "str",
                            "choices": ["enable", "disable"]},
                "internet-service": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "internet-service-custom": {"required": False, "type": "list",
                                            "options": {
                                                "name": {"required": True, "type": "str"}
                                            }},
                "internet-service-id": {"required": False, "type": "list",
                                        "options": {
                                            "id": {"required": True, "type": "int"}
                                        }},
                "internet-service-negate": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "internet-service-src": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "internet-service-src-custom": {"required": False, "type": "list",
                                                "options": {
                                                    "name": {"required": True, "type": "str"}
                                                }},
                "internet-service-src-id": {"required": False, "type": "list",
                                            "options": {
                                                "id": {"required": True, "type": "int"}
                                            }},
                "internet-service-src-negate": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "ippool": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "ips-sensor": {"required": False, "type": "str"},
                "label": {"required": False, "type": "str"},
                "learning-mode": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "logtraffic": {"required": False, "type": "str",
                               "choices": ["all", "utm", "disable"]},
                "logtraffic-start": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "match-vip": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "name": {"required": False, "type": "str"},
                "nat": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "natinbound": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "natip": {"required": False, "type": "str"},
                "natoutbound": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "ntlm": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "ntlm-enabled-browsers": {"required": False, "type": "list",
                                          "options": {
                                              "user-agent-string": {"required": True, "type": "str"}
                                          }},
                "ntlm-guest": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "outbound": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "per-ip-shaper": {"required": False, "type": "str"},
                "permit-any-host": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "permit-stun-host": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "policyid": {"required": True, "type": "int"},
                "poolname": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "profile-group": {"required": False, "type": "str"},
                "profile-protocol-options": {"required": False, "type": "str"},
                "profile-type": {"required": False, "type": "str",
                                 "choices": ["single", "group"]},
                "radius-mac-auth-bypass": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "redirect-url": {"required": False, "type": "str"},
                "replacemsg-override-group": {"required": False, "type": "str"},
                "rsso": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "rtp-addr": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "rtp-nat": {"required": False, "type": "str",
                            "choices": ["disable", "enable"]},
                "scan-botnet-connections": {"required": False, "type": "str",
                                            "choices": ["disable", "block", "monitor"]},
                "schedule": {"required": False, "type": "str"},
                "schedule-timeout": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "send-deny-packet": {"required": False, "type": "str",
                                     "choices": ["disable", "enable"]},
                "service": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "service-negate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "session-ttl": {"required": False, "type": "int"},
                "spamfilter-profile": {"required": False, "type": "str"},
                "srcaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "srcaddr-negate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "srcintf": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "ssh-filter-profile": {"required": False, "type": "str"},
                "ssl-mirror": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "ssl-mirror-intf": {"required": False, "type": "list",
                                    "options": {
                                        "name": {"required": True, "type": "str"}
                                    }},
                "ssl-ssh-profile": {"required": False, "type": "str"},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "tcp-mss-receiver": {"required": False, "type": "int"},
                "tcp-mss-sender": {"required": False, "type": "int"},
                "tcp-session-without-syn": {"required": False, "type": "str",
                                            "choices": ["all", "data-only", "disable"]},
                "timeout-send-rst": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "traffic-shaper": {"required": False, "type": "str"},
                "traffic-shaper-reverse": {"required": False, "type": "str"},
                "url-category": {"required": False, "type": "list",
                                 "options": {
                                     "id": {"required": True, "type": "int"}
                                 }},
                "users": {"required": False, "type": "list",
                          "options": {
                              "name": {"required": True, "type": "str"}
                          }},
                "utm-status": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "uuid": {"required": False, "type": "str"},
                "vlan-cos-fwd": {"required": False, "type": "int"},
                "vlan-cos-rev": {"required": False, "type": "int"},
                "vlan-filter": {"required": False, "type": "str"},
                "voip-profile": {"required": False, "type": "str"},
                "vpntunnel": {"required": False, "type": "str"},
                "waf-profile": {"required": False, "type": "str"},
                "wanopt": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "wanopt-detection": {"required": False, "type": "str",
                                     "choices": ["active", "passive", "off"]},
                "wanopt-passive-opt": {"required": False, "type": "str",
                                       "choices": ["default", "transparent", "non-transparent"]},
                "wanopt-peer": {"required": False, "type": "str"},
                "wanopt-profile": {"required": False, "type": "str"},
                "wccp": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "webcache": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "webcache-https": {"required": False, "type": "str",
                                   "choices": ["disable", "enable"]},
                "webfilter-profile": {"required": False, "type": "str"},
                "wsso": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: postgresql_ext
short_description: Add or remove PostgreSQL extensions from a database
description:
- Add or remove PostgreSQL extensions from a database.
version_added: '1.9'
options:
  name:
    description:
    - Name of the extension to add or remove.
    required: true
    type: str
    aliases:
    - ext
  db:
    description:
    - Name of the database to add or remove the extension to/from.
    required: true
    type: str
    aliases:
    - login_db
  schema:
    description:
    - Name of the schema to add the extension to.
    version_added: '2.8'
    type: str
  session_role:
    description:
    - Switch to session_role after connecting.
    - The specified session_role must be a role that the current login_user is a member of.
    - Permissions checking for SQL commands is carried out as though the session_role were the one that had logged in originally.
    type: str
    version_added: '2.8'
  state:
    description:
    - The database extension state.
    default: present
    choices: [ absent, present ]
    type: str
  cascade:
    description:
    - Automatically install/remove any extensions that this extension depends on
      that are not already installed/removed (supported since PostgreSQL 9.6).
    type: bool
    default: no
    version_added: '2.8'
  login_unix_socket:
    description:
      - Path to a Unix domain socket for local connections.
    type: str
    version_added: '2.8'
  ssl_mode:
    description:
      - Determines whether or with what priority a secure SSL TCP/IP connection will be negotiated with the server.
      - See https://www.postgresql.org/docs/current/static/libpq-ssl.html for more information on the modes.
      - Default of C(prefer) matches libpq default.
    type: str
    default: prefer
    choices: [ allow, disable, prefer, require, verify-ca, verify-full ]
    version_added: '2.8'
  ca_cert:
    description:
      - Specifies the name of a file containing SSL certificate authority (CA) certificate(s).
      - If the file exists, the server's certificate will be verified to be signed by one of these authorities.
    type: str
    aliases: [ ssl_rootcert ]
    version_added: '2.8'
notes:
- The default authentication assumes that you are either logging in as
  or sudo'ing to the C(postgres) account on the host.
- This module uses I(psycopg2), a Python PostgreSQL database adapter.
- You must ensure that psycopg2 is installed on the host before using this module.
- If the remote host is the PostgreSQL server (which is the default case),
  then PostgreSQL must also be installed on the remote host.
- For Ubuntu-based systems, install the C(postgresql), C(libpq-dev),
  and C(python-psycopg2) packages on the remote host before using this module.
requirements: [ psycopg2 ]
author:
- Daniel Schep (@dschep)
- Thomas O'Donnell (@andytom)
extends_documentation_fragment: postgres
'''

EXAMPLES = r'''
- name: Adds postgis extension to the database acme in the schema foo
  postgresql_ext:
    name: postgis
    db: acme
    schema: foo

- name: Removes postgis extension to the database acme
  postgresql_ext:
    name: postgis
    db: acme
    state: absent

- name: Adds earthdistance extension to the database template1 cascade
  postgresql_ext:
    name: earthdistance
    db: template1
    cascade: true

# In the example below, if earthdistance extension is installed,
# it will be removed too because it depends on cube:
- name: Removes cube extension from the database acme cascade
  postgresql_ext:
    name: cube
    db: acme
    cascade: yes
    state: absent
'''

RETURN = r'''
query:
  description: List of executed queries.
  returned: always
  type: list
  sample: ["DROP EXTENSION \"acme\""]

'''

import traceback

PSYCOPG2_IMP_ERR = None
try:
    import psycopg2
    import psycopg2.extras
    HAS_PSYCOPG2 = True
except ImportError:
    PSYCOPG2_IMP_ERR = traceback.format_exc()
    HAS_PSYCOPG2 = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.postgres import connect_to_db, postgres_common_argument_spec
from ansible.module_utils.six import iteritems
from ansible.module_utils._text import to_native
from ansible.module_utils.database import pg_quote_identifier

executed_queries = []


class NotSupportedError(Exception):
    pass


# ===========================================
# PostgreSQL module specific support methods.
#

def ext_exists(cursor, ext):
    query = "SELECT * FROM pg_extension WHERE extname=%(ext)s"
    cursor.execute(query, {'ext': ext})
    return cursor.rowcount == 1


def ext_delete(cursor, ext, cascade):
    if ext_exists(cursor, ext):
        query = "DROP EXTENSION \"%s\"" % ext
        if cascade:
            query += " CASCADE"
        cursor.execute(query)
        executed_queries.append(query)
        return True
    else:
        return False


def ext_create(cursor, ext, schema, cascade):
    if not ext_exists(cursor, ext):
        query = "CREATE EXTENSION \"%s\"" % ext
        if schema:
            query += " WITH SCHEMA \"%s\"" % schema
        if cascade:
            query += " CASCADE"
        cursor.execute(query)
        executed_queries.append(query)
        return True
    else:
        return False

# ===========================================
# Module execution.
#


def main():
    argument_spec = postgres_common_argument_spec()
    argument_spec.update(
        db=dict(type="str", required=True, aliases=["login_db"]),
        ext=dict(type="str", required=True, aliases=["name"]),
        schema=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present"]),
        cascade=dict(type="bool", default=False),
        session_role=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    if not HAS_PSYCOPG2:
        module.fail_json(msg=missing_required_lib('psycopg2'), exception=PSYCOPG2_IMP_ERR)

    db = module.params["db"]
    ext = module.params["ext"]
    schema = module.params["schema"]
    state = module.params["state"]
    cascade = module.params["cascade"]
    sslrootcert = module.params["ca_cert"]
    session_role = module.params["session_role"]
    changed = False

    # To use defaults values, keyword arguments must be absent, so
    # check which values are empty and don't include in the **kw
    # dictionary
    params_map = {
        "login_host": "host",
        "login_user": "user",
        "login_password": "password",
        "port": "port",
        "db": "database",
        "ssl_mode": "sslmode",
        "ca_cert": "sslrootcert"
    }
    kw = dict((params_map[k], v) for (k, v) in iteritems(module.params)
              if k in params_map and v != "" and v is not None)

    # If a login_unix_socket is specified, incorporate it here.
    is_localhost = "host" not in kw or kw["host"] == "" or kw["host"] == "localhost"
    if is_localhost and module.params["login_unix_socket"] != "":
        kw["host"] = module.params["login_unix_socket"]

    if psycopg2.__version__ < '2.4.3' and sslrootcert is not None:
        module.fail_json(msg='psycopg2 must be at least 2.4.3 in order to user the ca_cert parameter')

    db_connection = connect_to_db(module, kw, autocommit=True)
    cursor = db_connection.cursor(cursor_factory=psycopg2.extras.DictCursor)

    if session_role:
        try:
            cursor.execute('SET ROLE %s' % pg_quote_identifier(session_role, 'role'))
        except Exception as e:
            module.fail_json(msg="Could not switch role: %s" % to_native(e), exception=traceback.format_exc())

    try:
        if module.check_mode:
            if state == "present":
                changed = not ext_exists(cursor, ext)
            elif state == "absent":
                changed = ext_exists(cursor, ext)
        else:
            if state == "absent":
                changed = ext_delete(cursor, ext, cascade)

            elif state == "present":
                changed = ext_create(cursor, ext, schema, cascade)
    except NotSupportedError as e:
        module.fail_json(msg=to_native(e), exception=traceback.format_exc())
    except Exception as e:
        module.fail_json(msg="Database query failed: %s" % to_native(e), exception=traceback.format_exc())

    module.exit_json(changed=changed, db=db, ext=ext, queries=executed_queries)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_system_global
short_description: Configure global attributes in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify system feature and global category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    system_global:
        description:
            - Configure global attributes.
        default: null
        suboptions:
            admin-concurrent:
                description:
                    - Enable/disable concurrent administrator logins. (Use policy-auth-concurrent for firewall authenticated users.)
                choices:
                    - enable
                    - disable
            admin-console-timeout:
                description:
                    - Console login timeout that overrides the admintimeout value. (15 - 300 seconds) (15 seconds to 5 minutes). 0 the default, disables this
                       timeout.
            admin-https-pki-required:
                description:
                    - Enable/disable admin login method. Enable to force administrators to provide a valid certificate to log in if PKI is enabled. Disable to
                       allow administrators to log in with a certificate or password.
                choices:
                    - enable
                    - disable
            admin-https-ssl-versions:
                description:
                    - Allowed TLS versions for web administration.
                choices:
                    - tlsv1-0
                    - tlsv1-1
                    - tlsv1-2
            admin-lockout-duration:
                description:
                    - Amount of time in seconds that an administrator account is locked out after reaching the admin-lockout-threshold for repeated failed
                       login attempts.
            admin-lockout-threshold:
                description:
                    - Number of failed login attempts before an administrator account is locked out for the admin-lockout-duration.
            admin-login-max:
                description:
                    - Maximum number of administrators who can be logged in at the same time (1 - 100, default = 100)
            admin-maintainer:
                description:
                    - Enable/disable maintainer administrator login. When enabled, the maintainer account can be used to log in from the console after a hard
                       reboot. The password is "bcpb" followed by the FortiGate unit serial number. You have limited time to complete this login.
                choices:
                    - enable
                    - disable
            admin-port:
                description:
                    - Administrative access port for HTTP. (1 - 65535, default = 80).
            admin-restrict-local:
                description:
                    - Enable/disable local admin authentication restriction when remote authenticator is up and running. (default = disable)
                choices:
                    - enable
                    - disable
            admin-scp:
                description:
                    - Enable/disable using SCP to download the system configuration. You can use SCP as an alternative method for backing up the configuration.
                choices:
                    - enable
                    - disable
            admin-server-cert:
                description:
                    - Server certificate that the FortiGate uses for HTTPS administrative connections. Source certificate.local.name.
            admin-sport:
                description:
                    - Administrative access port for HTTPS. (1 - 65535, default = 443).
            admin-ssh-grace-time:
                description:
                    - Maximum time in seconds permitted between making an SSH connection to the FortiGate unit and authenticating (10 - 3600 sec (1 hour),
                       default 120).
            admin-ssh-password:
                description:
                    - Enable/disable password authentication for SSH admin access.
                choices:
                    - enable
                    - disable
            admin-ssh-port:
                description:
                    - Administrative access port for SSH. (1 - 65535, default = 22).
            admin-ssh-v1:
                description:
                    - Enable/disable SSH v1 compatibility.
                choices:
                    - enable
                    - disable
            admin-telnet-port:
                description:
                    - Administrative access port for TELNET. (1 - 65535, default = 23).
            admintimeout:
                description:
                    - Number of minutes before an idle administrator session times out (5 - 480 minutes (8 hours), default = 5). A shorter idle timeout is
                       more secure.
            alias:
                description:
                    - Alias for your FortiGate unit.
            allow-traffic-redirect:
                description:
                    - Disable to allow traffic to be routed back on a different interface.
                choices:
                    - enable
                    - disable
            anti-replay:
                description:
                    - Level of checking for packet replay and TCP sequence checking.
                choices:
                    - disable
                    - loose
                    - strict
            arp-max-entry:
                description:
                    - Maximum number of dynamically learned MAC addresses that can be added to the ARP table (131072 - 2147483647, default = 131072).
            asymroute:
                description:
                    - Enable/disable asymmetric route.
                choices:
                    - enable
                    - disable
            auth-cert:
                description:
                    - Server certificate that the FortiGate uses for HTTPS firewall authentication connections. Source certificate.local.name.
            auth-http-port:
                description:
                    - User authentication HTTP port. (1 - 65535, default = 80).
            auth-https-port:
                description:
                    - User authentication HTTPS port. (1 - 65535, default = 443).
            auth-keepalive:
                description:
                    - Enable to prevent user authentication sessions from timing out when idle.
                choices:
                    - enable
                    - disable
            auth-session-limit:
                description:
                    - Action to take when the number of allowed user authenticated sessions is reached.
                choices:
                    - block-new
                    - logout-inactive
            auto-auth-extension-device:
                description:
                    - Enable/disable automatic authorization of dedicated Fortinet extension devices.
                choices:
                    - enable
                    - disable
            av-affinity:
                description:
                    - Affinity setting for AV scanning (hexadecimal value up to 256 bits in the format of xxxxxxxxxxxxxxxx).
            av-failopen:
                description:
                    - Set the action to take if the FortiGate is running low on memory or the proxy connection limit has been reached.
                choices:
                    - pass
                    - off
                    - one-shot
            av-failopen-session:
                description:
                    - When enabled and a proxy for a protocol runs out of room in its session table, that protocol goes into failopen mode and enacts the
                       action specified by av-failopen.
                choices:
                    - enable
                    - disable
            batch-cmdb:
                description:
                    - Enable/disable batch mode, allowing you to enter a series of CLI commands that will execute as a group once they are loaded.
                choices:
                    - enable
                    - disable
            block-session-timer:
                description:
                    - Duration in seconds for blocked sessions (1 - 300 sec  (5 minutes), default = 30).
            br-fdb-max-entry:
                description:
                    - Maximum number of bridge forwarding database (FDB) entries.
            cert-chain-max:
                description:
                    - Maximum number of certificates that can be traversed in a certificate chain.
            cfg-revert-timeout:
                description:
                    - Time-out for reverting to the last saved configuration.
            cfg-save:
                description:
                    - Configuration file save mode for CLI changes.
                choices:
                    - automatic
                    - manual
                    - revert
            check-protocol-header:
                description:
                    - Level of checking performed on protocol headers. Strict checking is more thorough but may affect performance. Loose checking is ok in
                       most cases.
                choices:
                    - loose
                    - strict
            check-reset-range:
                description:
                    - Configure ICMP error message verification. You can either apply strict RST range checking or disable it.
                choices:
                    - strict
                    - disable
            cli-audit-log:
                description:
                    - Enable/disable CLI audit log.
                choices:
                    - enable
                    - disable
            clt-cert-req:
                description:
                    - Enable/disable requiring administrators to have a client certificate to log into the GUI using HTTPS.
                choices:
                    - enable
                    - disable
            compliance-check:
                description:
                    - Enable/disable global PCI DSS compliance check.
                choices:
                    - enable
                    - disable
            compliance-check-time:
                description:
                    - Time of day to run scheduled PCI DSS compliance checks.
            cpu-use-threshold:
                description:
                    - Threshold at which CPU usage is reported. (% of total CPU, default = 90).
            csr-ca-attribute:
                description:
                    - Enable/disable the CA attribute in certificates. Some CA servers reject CSRs that have the CA attribute.
                choices:
                    - enable
                    - disable
            daily-restart:
                description:
                    - Enable/disable daily restart of FortiGate unit. Use the restart-time option to set the time of day for the restart.
                choices:
                    - enable
                    - disable
            device-identification-active-scan-delay:
                description:
                    - Number of seconds to passively scan a device before performing an active scan. (20 - 3600 sec, (20 sec to 1 hour), default = 90).
            device-idle-timeout:
                description:
                    - Time in seconds that a device must be idle to automatically log the device user out. (30 - 31536000 sec (30 sec to 1 year), default =
                       300).
            dh-params:
                description:
                    - Number of bits to use in the Diffie-Hellman exchange for HTTPS/SSH protocols.
                choices:
                    - 1024
                    - 1536
                    - 2048
                    - 3072
                    - 4096
                    - 6144
                    - 8192
            dst:
                description:
                    - Enable/disable daylight saving time.
                choices:
                    - enable
                    - disable
            endpoint-control-fds-access:
                description:
                    - Enable/disable access to the FortiGuard network for non-compliant endpoints.
                choices:
                    - enable
                    - disable
            endpoint-control-portal-port:
                description:
                    - Endpoint control portal port (1 - 65535).
            failtime:
                description:
                    - Fail-time for server lost.
            fds-statistics:
                description:
                    - Enable/disable sending IPS, Application Control, and AntiVirus data to FortiGuard. This data is used to improve FortiGuard services and
                       is not shared with external parties and is protected by Fortinet's privacy policy.
                choices:
                    - enable
                    - disable
            fds-statistics-period:
                description:
                    - FortiGuard statistics collection period in minutes. (1 - 1440 min (1 min to 24 hours), default = 60).
            fgd-alert-subscription:
                description:
                    - Type of alert to retrieve from FortiGuard.
                choices:
                    - advisory
                    - latest-threat
                    - latest-virus
                    - latest-attack
                    - new-antivirus-db
                    - new-attack-db
            fortiextender:
                description:
                    - Enable/disable FortiExtender.
                choices:
                    - enable
                    - disable
            fortiextender-data-port:
                description:
                    - FortiExtender data port (1024 - 49150, default = 25246).
            fortiextender-vlan-mode:
                description:
                    - Enable/disable FortiExtender VLAN mode.
                choices:
                    - enable
                    - disable
            fortiservice-port:
                description:
                    - FortiService port (1 - 65535, default = 8013). Used by FortiClient endpoint compliance. Older versions of FortiClient used a different
                       port.
            gui-certificates:
                description:
                    - Enable/disable the System > Certificate GUI page, allowing you to add and configure certificates from the GUI.
                choices:
                    - enable
                    - disable
            gui-custom-language:
                description:
                    - Enable/disable custom languages in GUI.
                choices:
                    - enable
                    - disable
            gui-date-format:
                description:
                    - Default date format used throughout GUI.
                choices:
                    - yyyy/MM/dd
                    - dd/MM/yyyy
                    - MM/dd/yyyy
                    - yyyy-MM-dd
                    - dd-MM-yyyy
                    - MM-dd-yyyy
            gui-device-latitude:
                description:
                    - Add the latitude of the location of this FortiGate to position it on the Threat Map.
            gui-device-longitude:
                description:
                    - Add the longitude of the location of this FortiGate to position it on the Threat Map.
            gui-display-hostname:
                description:
                    - Enable/disable displaying the FortiGate's hostname on the GUI login page.
                choices:
                    - enable
                    - disable
            gui-ipv6:
                description:
                    - Enable/disable IPv6 settings on the GUI.
                choices:
                    - enable
                    - disable
            gui-lines-per-page:
                description:
                    - Number of lines to display per page for web administration.
            gui-theme:
                description:
                    - Color scheme for the administration GUI.
                choices:
                    - green
                    - red
                    - blue
                    - melongene
                    - mariner
            gui-wireless-opensecurity:
                description:
                    - Enable/disable wireless open security option on the GUI.
                choices:
                    - enable
                    - disable
            honor-df:
                description:
                    - Enable/disable honoring of Don't-Fragment (DF) flag.
                choices:
                    - enable
                    - disable
            hostname:
                description:
                    - FortiGate unit's hostname. Most models will truncate names longer than 24 characters. Some models support hostnames up to 35 characters.
            igmp-state-limit:
                description:
                    - Maximum number of IGMP memberships (96 - 64000, default = 3200).
            interval:
                description:
                    - Dead gateway detection interval.
            ip-src-port-range:
                description:
                    - IP source port range used for traffic originating from the FortiGate unit.
            ips-affinity:
                description:
                    - Affinity setting for IPS (hexadecimal value up to 256 bits in the format of xxxxxxxxxxxxxxxx; allowed CPUs must be less than total
                       number of IPS engine daemons).
            ipsec-asic-offload:
                description:
                    - Enable/disable ASIC offloading (hardware acceleration) for IPsec VPN traffic. Hardware acceleration can offload IPsec VPN sessions and
                       accelerate encryption and decryption.
                choices:
                    - enable
                    - disable
            ipsec-hmac-offload:
                description:
                    - Enable/disable offloading (hardware acceleration) of HMAC processing for IPsec VPN.
                choices:
                    - enable
                    - disable
            ipsec-soft-dec-async:
                description:
                    - Enable/disable software decryption asynchronization (using multiple CPUs to do decryption) for IPsec VPN traffic.
                choices:
                    - enable
                    - disable
            ipv6-accept-dad:
                description:
                    - Enable/disable acceptance of IPv6 Duplicate Address Detection (DAD).
            ipv6-allow-anycast-probe:
                description:
                    - Enable/disable IPv6 address probe through Anycast.
                choices:
                    - enable
                    - disable
            language:
                description:
                    - GUI display language.
                choices:
                    - english
                    - french
                    - spanish
                    - portuguese
                    - japanese
                    - trach
                    - simch
                    - korean
            ldapconntimeout:
                description:
                    - Global timeout for connections with remote LDAP servers in milliseconds (0 - 4294967295, default 500).
            lldp-transmission:
                description:
                    - Enable/disable Link Layer Discovery Protocol (LLDP) transmission.
                choices:
                    - enable
                    - disable
            log-ssl-connection:
                description:
                    - Enable/disable logging of SSL connection events.
                choices:
                    - enable
                    - disable
            log-uuid:
                description:
                    - Whether UUIDs are added to traffic logs. You can disable UUIDs, add firewall policy UUIDs to traffic logs, or add all UUIDs to traffic
                       logs.
                choices:
                    - disable
                    - policy-only
                    - extended
            login-timestamp:
                description:
                    - Enable/disable login time recording.
                choices:
                    - enable
                    - disable
            long-vdom-name:
                description:
                    - Enable/disable long VDOM name support.
                choices:
                    - enable
                    - disable
            management-vdom:
                description:
                    - Management virtual domain name. Source system.vdom.name.
            max-dlpstat-memory:
                description:
                    - Maximum DLP stat memory (0 - 4294967295).
            max-route-cache-size:
                description:
                    - Maximum number of IP route cache entries (0 - 2147483647).
            mc-ttl-notchange:
                description:
                    - Enable/disable no modification of multicast TTL.
                choices:
                    - enable
                    - disable
            memory-use-threshold-extreme:
                description:
                    - Threshold at which memory usage is considered extreme (new sessions are dropped) (% of total RAM, default = 95).
            memory-use-threshold-green:
                description:
                    - Threshold at which memory usage forces the FortiGate to exit conserve mode (% of total RAM, default = 82).
            memory-use-threshold-red:
                description:
                    - Threshold at which memory usage forces the FortiGate to enter conserve mode (% of total RAM, default = 88).
            miglog-affinity:
                description:
                    - Affinity setting for logging (64-bit hexadecimal value in the format of xxxxxxxxxxxxxxxx).
            miglogd-children:
                description:
                    - Number of logging (miglogd) processes to be allowed to run. Higher number can reduce performance; lower number can slow log processing
                       time. No logs will be dropped or lost if the number is changed.
            multi-factor-authentication:
                description:
                    - Enforce all login methods to require an additional authentication factor (default = optional).
                choices:
                    - optional
                    - mandatory
            multicast-forward:
                description:
                    - Enable/disable multicast forwarding.
                choices:
                    - enable
                    - disable
            ndp-max-entry:
                description:
                    - Maximum number of NDP table entries (set to 65,536 or higher; if set to 0, kernel holds 65,536 entries).
            per-user-bwl:
                description:
                    - Enable/disable per-user black/white list filter.
                choices:
                    - enable
                    - disable
            policy-auth-concurrent:
                description:
                    - Number of concurrent firewall use logins from the same user (1 - 100, default = 0 means no limit).
            post-login-banner:
                description:
                    - Enable/disable displaying the administrator access disclaimer message after an administrator successfully logs in.
                choices:
                    - disable
                    - enable
            pre-login-banner:
                description:
                    - Enable/disable displaying the administrator access disclaimer message on the login page before an administrator logs in.
                choices:
                    - enable
                    - disable
            private-data-encryption:
                description:
                    - Enable/disable private data encryption using an AES 128-bit key.
                choices:
                    - disable
                    - enable
            proxy-auth-lifetime:
                description:
                    - Enable/disable authenticated users lifetime control.  This is a cap on the total time a proxy user can be authenticated for after which
                       re-authentication will take place.
                choices:
                    - enable
                    - disable
            proxy-auth-lifetime-timeout:
                description:
                    - Lifetime timeout in minutes for authenticated users (5  - 65535 min, default=480 (8 hours)).
            proxy-auth-timeout:
                description:
                    - Authentication timeout in minutes for authenticated users (1 - 3600 sec, default = 300).
            proxy-cipher-hardware-acceleration:
                description:
                    - Enable/disable using content processor (CP8 or CP9) hardware acceleration to encrypt and decrypt IPsec and SSL traffic.
                choices:
                    - disable
                    - enable
            proxy-kxp-hardware-acceleration:
                description:
                    - Enable/disable using the content processor to accelerate KXP traffic.
                choices:
                    - disable
                    - enable
            proxy-re-authentication-mode:
                description:
                    - Control if users must re-authenticate after a session is closed, traffic has been idle, or from the point at which the user was first
                       created.
                choices:
                    - session
                    - traffic
                    - absolute
            proxy-worker-count:
                description:
                    - Proxy worker count.
            radius-port:
                description:
                    - RADIUS service port number.
            reboot-upon-config-restore:
                description:
                    - Enable/disable reboot of system upon restoring configuration.
                choices:
                    - enable
                    - disable
            refresh:
                description:
                    - Statistics refresh interval in GUI.
            remoteauthtimeout:
                description:
                    - Number of seconds that the FortiGate waits for responses from remote RADIUS, LDAP, or TACACS+ authentication servers. (0-300 sec,
                       default = 5, 0 means no timeout).
            reset-sessionless-tcp:
                description:
                    - Action to perform if the FortiGate receives a TCP packet but cannot find a corresponding session in its session table. NAT/Route mode
                       only.
                choices:
                    - enable
                    - disable
            restart-time:
                description:
                    - "Daily restart time (hh:mm)."
            revision-backup-on-logout:
                description:
                    - Enable/disable back-up of the latest configuration revision when an administrator logs out of the CLI or GUI.
                choices:
                    - enable
                    - disable
            revision-image-auto-backup:
                description:
                    - Enable/disable back-up of the latest configuration revision after the firmware is upgraded.
                choices:
                    - enable
                    - disable
            scanunit-count:
                description:
                    - Number of scanunits. The range and the default depend on the number of CPUs. Only available on FortiGate units with multiple CPUs.
            security-rating-result-submission:
                description:
                    - Enable/disable the submission of Security Rating results to FortiGuard.
                choices:
                    - enable
                    - disable
            security-rating-run-on-schedule:
                description:
                    - Enable/disable scheduled runs of Security Rating.
                choices:
                    - enable
                    - disable
            send-pmtu-icmp:
                description:
                    - Enable/disable sending of path maximum transmission unit (PMTU) - ICMP destination unreachable packet and to support PMTUD protocol on
                       your network to reduce fragmentation of packets.
                choices:
                    - enable
                    - disable
            snat-route-change:
                description:
                    - Enable/disable the ability to change the static NAT route.
                choices:
                    - enable
                    - disable
            special-file-23-support:
                description:
                    - Enable/disable IPS detection of HIBUN format files when using Data Leak Protection.
                choices:
                    - disable
                    - enable
            ssh-cbc-cipher:
                description:
                    - Enable/disable CBC cipher for SSH access.
                choices:
                    - enable
                    - disable
            ssh-hmac-md5:
                description:
                    - Enable/disable HMAC-MD5 for SSH access.
                choices:
                    - enable
                    - disable
            ssh-kex-sha1:
                description:
                    - Enable/disable SHA1 key exchange for SSH access.
                choices:
                    - enable
                    - disable
            ssl-min-proto-version:
                description:
                    - Minimum supported protocol version for SSL/TLS connections (default = TLSv1.2).
                choices:
                    - SSLv3
                    - TLSv1
                    - TLSv1-1
                    - TLSv1-2
            ssl-static-key-ciphers:
                description:
                    - Enable/disable static key ciphers in SSL/TLS connections (e.g. AES128-SHA, AES256-SHA, AES128-SHA256, AES256-SHA256).
                choices:
                    - enable
                    - disable
            sslvpn-cipher-hardware-acceleration:
                description:
                    - Enable/disable SSL VPN hardware acceleration.
                choices:
                    - enable
                    - disable
            sslvpn-kxp-hardware-acceleration:
                description:
                    - Enable/disable SSL VPN KXP hardware acceleration.
                choices:
                    - enable
                    - disable
            sslvpn-max-worker-count:
                description:
                    - Maximum number of SSL VPN processes. Upper limit for this value is the number of CPUs and depends on the model.
            sslvpn-plugin-version-check:
                description:
                    - Enable/disable checking browser's plugin version by SSL VPN.
                choices:
                    - enable
                    - disable
            strict-dirty-session-check:
                description:
                    - Enable to check the session against the original policy when revalidating. This can prevent dropping of redirected sessions when
                       web-filtering and authentication are enabled together. If this option is enabled, the FortiGate unit deletes a session if a routing or
                          policy change causes the session to no longer match the policy that originally allowed the session.
                choices:
                    - enable
                    - disable
            strong-crypto:
                description:
                    - Enable to use strong encryption and only allow strong ciphers (AES, 3DES) and digest (SHA1) for HTTPS/SSH/TLS/SSL functions.
                choices:
                    - enable
                    - disable
            switch-controller:
                description:
                    - Enable/disable switch controller feature. Switch controller allows you to manage FortiSwitch from the FortiGate itself.
                choices:
                    - disable
                    - enable
            switch-controller-reserved-network:
                description:
                    - Enable reserved network subnet for controlled switches. This is available when the switch controller is enabled.
            sys-perf-log-interval:
                description:
                    - Time in minutes between updates of performance statistics logging. (1 - 15 min, default = 5, 0 = disabled).
            tcp-halfclose-timer:
                description:
                    - Number of seconds the FortiGate unit should wait to close a session after one peer has sent a FIN packet but the other has not responded
                       (1 - 86400 sec (1 day), default = 120).
            tcp-halfopen-timer:
                description:
                    - Number of seconds the FortiGate unit should wait to close a session after one peer has sent an open session packet but the other has not
                       responded (1 - 86400 sec (1 day), default = 10).
            tcp-option:
                description:
                    - Enable SACK, timestamp and MSS TCP options.
                choices:
                    - enable
                    - disable
            tcp-timewait-timer:
                description:
                    - Length of the TCP TIME-WAIT state in seconds.
            tftp:
                description:
                    - Enable/disable TFTP.
                choices:
                    - enable
                    - disable
            timezone:
                description:
                    - Number corresponding to your time zone from 00 to 86. Enter set timezone ? to view the list of time zones and the numbers that represent
                       them.
                choices:
                    - 01
                    - 02
                    - 03
                    - 04
                    - 05
                    - 81
                    - 06
                    - 07
                    - 08
                    - 09
                    - 10
                    - 11
                    - 12
                    - 13
                    - 74
                    - 14
                    - 77
                    - 15
                    - 87
                    - 16
                    - 17
                    - 18
                    - 19
                    - 20
                    - 75
                    - 21
                    - 22
                    - 23
                    - 24
                    - 80
                    - 79
                    - 25
                    - 26
                    - 27
                    - 28
                    - 78
                    - 29
                    - 30
                    - 31
                    - 32
                    - 33
                    - 34
                    - 35
                    - 36
                    - 37
                    - 38
                    - 83
                    - 84
                    - 40
                    - 85
                    - 41
                    - 42
                    - 43
                    - 39
                    - 44
                    - 46
                    - 47
                    - 51
                    - 48
                    - 45
                    - 49
                    - 50
                    - 52
                    - 53
                    - 54
                    - 55
                    - 56
                    - 57
                    - 58
                    - 59
                    - 60
                    - 62
                    - 63
                    - 61
                    - 64
                    - 65
                    - 66
                    - 67
                    - 68
                    - 69
                    - 70
                    - 71
                    - 72
                    - 00
                    - 82
                    - 73
                    - 86
                    - 76
            tp-mc-skip-policy:
                description:
                    - Enable/disable skip policy check and allow multicast through.
                choices:
                    - enable
                    - disable
            traffic-priority:
                description:
                    - Choose Type of Service (ToS) or Differentiated Services Code Point (DSCP) for traffic prioritization in traffic shaping.
                choices:
                    - tos
                    - dscp
            traffic-priority-level:
                description:
                    - Default system-wide level of priority for traffic prioritization.
                choices:
                    - low
                    - medium
                    - high
            two-factor-email-expiry:
                description:
                    - Email-based two-factor authentication session timeout (30 - 300 seconds (5 minutes), default = 60).
            two-factor-fac-expiry:
                description:
                    - FortiAuthenticator token authentication session timeout (10 - 3600 seconds (1 hour), default = 60).
            two-factor-ftk-expiry:
                description:
                    - FortiToken authentication session timeout (60 - 600 sec (10 minutes), default = 60).
            two-factor-ftm-expiry:
                description:
                    - FortiToken Mobile session timeout (1 - 168 hours (7 days), default = 72).
            two-factor-sms-expiry:
                description:
                    - SMS-based two-factor authentication session timeout (30 - 300 sec, default = 60).
            udp-idle-timer:
                description:
                    - UDP connection session timeout. This command can be useful in managing CPU and memory resources (1 - 86400 seconds (1 day), default =
                       60).
            user-server-cert:
                description:
                    - Certificate to use for https user authentication. Source certificate.local.name.
            vdom-admin:
                description:
                    - Enable/disable support for multiple virtual domains (VDOMs).
                choices:
                    - enable
                    - disable
            vip-arp-range:
                description:
                    - Controls the number of ARPs that the FortiGate sends for a Virtual IP (VIP) address range.
                choices:
                    - unlimited
                    - restricted
            virtual-server-count:
                description:
                    - Maximum number of virtual server processes to create. The maximum is the number of CPU cores. This is not available on single-core CPUs.
            virtual-server-hardware-acceleration:
                description:
                    - Enable/disable virtual server hardware acceleration.
                choices:
                    - disable
                    - enable
            wad-affinity:
                description:
                    - Affinity setting for wad (hexadecimal value up to 256 bits in the format of xxxxxxxxxxxxxxxx).
            wad-csvc-cs-count:
                description:
                    - Number of concurrent WAD-cache-service object-cache processes.
            wad-csvc-db-count:
                description:
                    - Number of concurrent WAD-cache-service byte-cache processes.
            wad-source-affinity:
                description:
                    - Enable/disable dispatching traffic to WAD workers based on source affinity.
                choices:
                    - disable
                    - enable
            wad-worker-count:
                description:
                    - Number of explicit proxy WAN optimization daemon (WAD) processes. By default WAN optimization, explicit proxy, and web caching is
                       handled by all of the CPU cores in a FortiGate unit.
            wifi-ca-certificate:
                description:
                    - CA certificate that verifies the WiFi certificate. Source certificate.ca.name.
            wifi-certificate:
                description:
                    - Certificate to use for WiFi authentication. Source certificate.local.name.
            wimax-4g-usb:
                description:
                    - Enable/disable comparability with WiMAX 4G USB devices.
                choices:
                    - enable
                    - disable
            wireless-controller:
                description:
                    - Enable/disable the wireless controller feature to use the FortiGate unit to manage FortiAPs.
                choices:
                    - enable
                    - disable
            wireless-controller-port:
                description:
                    - Port used for the control channel in wireless controller mode (wireless-mode is ac). The data channel port is the control channel port
                       number plus one (1024 - 49150, default = 5246).
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure global attributes.
    fortios_system_global:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      system_global:
        admin-concurrent: "enable"
        admin-console-timeout: "4"
        admin-https-pki-required: "enable"
        admin-https-ssl-versions: "tlsv1-0"
        admin-lockout-duration: "7"
        admin-lockout-threshold: "8"
        admin-login-max: "9"
        admin-maintainer: "enable"
        admin-port: "11"
        admin-restrict-local: "enable"
        admin-scp: "enable"
        admin-server-cert: "<your_own_value> (source certificate.local.name)"
        admin-sport: "15"
        admin-ssh-grace-time: "16"
        admin-ssh-password: "enable"
        admin-ssh-port: "18"
        admin-ssh-v1: "enable"
        admin-telnet-port: "20"
        admintimeout: "21"
        alias: "<your_own_value>"
        allow-traffic-redirect: "enable"
        anti-replay: "disable"
        arp-max-entry: "25"
        asymroute: "enable"
        auth-cert: "<your_own_value> (source certificate.local.name)"
        auth-http-port: "28"
        auth-https-port: "29"
        auth-keepalive: "enable"
        auth-session-limit: "block-new"
        auto-auth-extension-device: "enable"
        av-affinity: "<your_own_value>"
        av-failopen: "pass"
        av-failopen-session: "enable"
        batch-cmdb: "enable"
        block-session-timer: "37"
        br-fdb-max-entry: "38"
        cert-chain-max: "39"
        cfg-revert-timeout: "40"
        cfg-save: "automatic"
        check-protocol-header: "loose"
        check-reset-range: "strict"
        cli-audit-log: "enable"
        clt-cert-req: "enable"
        compliance-check: "enable"
        compliance-check-time: "<your_own_value>"
        cpu-use-threshold: "48"
        csr-ca-attribute: "enable"
        daily-restart: "enable"
        device-identification-active-scan-delay: "51"
        device-idle-timeout: "52"
        dh-params: "1024"
        dst: "enable"
        endpoint-control-fds-access: "enable"
        endpoint-control-portal-port: "56"
        failtime: "57"
        fds-statistics: "enable"
        fds-statistics-period: "59"
        fgd-alert-subscription: "advisory"
        fortiextender: "enable"
        fortiextender-data-port: "62"
        fortiextender-vlan-mode: "enable"
        fortiservice-port: "64"
        gui-certificates: "enable"
        gui-custom-language: "enable"
        gui-date-format: "yyyy/MM/dd"
        gui-device-latitude: "<your_own_value>"
        gui-device-longitude: "<your_own_value>"
        gui-display-hostname: "enable"
        gui-ipv6: "enable"
        gui-lines-per-page: "72"
        gui-theme: "green"
        gui-wireless-opensecurity: "enable"
        honor-df: "enable"
        hostname: "myhostname"
        igmp-state-limit: "77"
        interval: "78"
        ip-src-port-range: "<your_own_value>"
        ips-affinity: "<your_own_value>"
        ipsec-asic-offload: "enable"
        ipsec-hmac-offload: "enable"
        ipsec-soft-dec-async: "enable"
        ipv6-accept-dad: "84"
        ipv6-allow-anycast-probe: "enable"
        language: "english"
        ldapconntimeout: "87"
        lldp-transmission: "enable"
        log-ssl-connection: "enable"
        log-uuid: "disable"
        login-timestamp: "enable"
        long-vdom-name: "enable"
        management-vdom: "<your_own_value> (source system.vdom.name)"
        max-dlpstat-memory: "94"
        max-route-cache-size: "95"
        mc-ttl-notchange: "enable"
        memory-use-threshold-extreme: "97"
        memory-use-threshold-green: "98"
        memory-use-threshold-red: "99"
        miglog-affinity: "<your_own_value>"
        miglogd-children: "101"
        multi-factor-authentication: "optional"
        multicast-forward: "enable"
        ndp-max-entry: "104"
        per-user-bwl: "enable"
        policy-auth-concurrent: "106"
        post-login-banner: "disable"
        pre-login-banner: "enable"
        private-data-encryption: "disable"
        proxy-auth-lifetime: "enable"
        proxy-auth-lifetime-timeout: "111"
        proxy-auth-timeout: "112"
        proxy-cipher-hardware-acceleration: "disable"
        proxy-kxp-hardware-acceleration: "disable"
        proxy-re-authentication-mode: "session"
        proxy-worker-count: "116"
        radius-port: "117"
        reboot-upon-config-restore: "enable"
        refresh: "119"
        remoteauthtimeout: "120"
        reset-sessionless-tcp: "enable"
        restart-time: "<your_own_value>"
        revision-backup-on-logout: "enable"
        revision-image-auto-backup: "enable"
        scanunit-count: "125"
        security-rating-result-submission: "enable"
        security-rating-run-on-schedule: "enable"
        send-pmtu-icmp: "enable"
        snat-route-change: "enable"
        special-file-23-support: "disable"
        ssh-cbc-cipher: "enable"
        ssh-hmac-md5: "enable"
        ssh-kex-sha1: "enable"
        ssl-min-proto-version: "SSLv3"
        ssl-static-key-ciphers: "enable"
        sslvpn-cipher-hardware-acceleration: "enable"
        sslvpn-kxp-hardware-acceleration: "enable"
        sslvpn-max-worker-count: "138"
        sslvpn-plugin-version-check: "enable"
        strict-dirty-session-check: "enable"
        strong-crypto: "enable"
        switch-controller: "disable"
        switch-controller-reserved-network: "<your_own_value>"
        sys-perf-log-interval: "144"
        tcp-halfclose-timer: "145"
        tcp-halfopen-timer: "146"
        tcp-option: "enable"
        tcp-timewait-timer: "148"
        tftp: "enable"
        timezone: "01"
        tp-mc-skip-policy: "enable"
        traffic-priority: "tos"
        traffic-priority-level: "low"
        two-factor-email-expiry: "154"
        two-factor-fac-expiry: "155"
        two-factor-ftk-expiry: "156"
        two-factor-ftm-expiry: "157"
        two-factor-sms-expiry: "158"
        udp-idle-timer: "159"
        user-server-cert: "<your_own_value> (source certificate.local.name)"
        vdom-admin: "enable"
        vip-arp-range: "unlimited"
        virtual-server-count: "163"
        virtual-server-hardware-acceleration: "disable"
        wad-affinity: "<your_own_value>"
        wad-csvc-cs-count: "166"
        wad-csvc-db-count: "167"
        wad-source-affinity: "disable"
        wad-worker-count: "169"
        wifi-ca-certificate: "<your_own_value> (source certificate.ca.name)"
        wifi-certificate: "<your_own_value> (source certificate.local.name)"
        wimax-4g-usb: "enable"
        wireless-controller: "enable"
        wireless-controller-port: "174"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_system_global_data(json):
    option_list = ['admin-concurrent', 'admin-console-timeout', 'admin-https-pki-required',
                   'admin-https-ssl-versions', 'admin-lockout-duration', 'admin-lockout-threshold',
                   'admin-login-max', 'admin-maintainer', 'admin-port',
                   'admin-restrict-local', 'admin-scp', 'admin-server-cert',
                   'admin-sport', 'admin-ssh-grace-time', 'admin-ssh-password',
                   'admin-ssh-port', 'admin-ssh-v1', 'admin-telnet-port',
                   'admintimeout', 'alias', 'allow-traffic-redirect',
                   'anti-replay', 'arp-max-entry', 'asymroute',
                   'auth-cert', 'auth-http-port', 'auth-https-port',
                   'auth-keepalive', 'auth-session-limit', 'auto-auth-extension-device',
                   'av-affinity', 'av-failopen', 'av-failopen-session',
                   'batch-cmdb', 'block-session-timer', 'br-fdb-max-entry',
                   'cert-chain-max', 'cfg-revert-timeout', 'cfg-save',
                   'check-protocol-header', 'check-reset-range', 'cli-audit-log',
                   'clt-cert-req', 'compliance-check', 'compliance-check-time',
                   'cpu-use-threshold', 'csr-ca-attribute', 'daily-restart',
                   'device-identification-active-scan-delay', 'device-idle-timeout', 'dh-params',
                   'dst', 'endpoint-control-fds-access', 'endpoint-control-portal-port',
                   'failtime', 'fds-statistics', 'fds-statistics-period',
                   'fgd-alert-subscription', 'fortiextender', 'fortiextender-data-port',
                   'fortiextender-vlan-mode', 'fortiservice-port', 'gui-certificates',
                   'gui-custom-language', 'gui-date-format', 'gui-device-latitude',
                   'gui-device-longitude', 'gui-display-hostname', 'gui-ipv6',
                   'gui-lines-per-page', 'gui-theme', 'gui-wireless-opensecurity',
                   'honor-df', 'hostname', 'igmp-state-limit',
                   'interval', 'ip-src-port-range', 'ips-affinity',
                   'ipsec-asic-offload', 'ipsec-hmac-offload', 'ipsec-soft-dec-async',
                   'ipv6-accept-dad', 'ipv6-allow-anycast-probe', 'language',
                   'ldapconntimeout', 'lldp-transmission', 'log-ssl-connection',
                   'log-uuid', 'login-timestamp', 'long-vdom-name',
                   'management-vdom', 'max-dlpstat-memory', 'max-route-cache-size',
                   'mc-ttl-notchange', 'memory-use-threshold-extreme', 'memory-use-threshold-green',
                   'memory-use-threshold-red', 'miglog-affinity', 'miglogd-children',
                   'multi-factor-authentication', 'multicast-forward', 'ndp-max-entry',
                   'per-user-bwl', 'policy-auth-concurrent', 'post-login-banner',
                   'pre-login-banner', 'private-data-encryption', 'proxy-auth-lifetime',
                   'proxy-auth-lifetime-timeout', 'proxy-auth-timeout', 'proxy-cipher-hardware-acceleration',
                   'proxy-kxp-hardware-acceleration', 'proxy-re-authentication-mode', 'proxy-worker-count',
                   'radius-port', 'reboot-upon-config-restore', 'refresh',
                   'remoteauthtimeout', 'reset-sessionless-tcp', 'restart-time',
                   'revision-backup-on-logout', 'revision-image-auto-backup', 'scanunit-count',
                   'security-rating-result-submission', 'security-rating-run-on-schedule', 'send-pmtu-icmp',
                   'snat-route-change', 'special-file-23-support', 'ssh-cbc-cipher',
                   'ssh-hmac-md5', 'ssh-kex-sha1', 'ssl-min-proto-version',
                   'ssl-static-key-ciphers', 'sslvpn-cipher-hardware-acceleration', 'sslvpn-kxp-hardware-acceleration',
                   'sslvpn-max-worker-count', 'sslvpn-plugin-version-check', 'strict-dirty-session-check',
                   'strong-crypto', 'switch-controller', 'switch-controller-reserved-network',
                   'sys-perf-log-interval', 'tcp-halfclose-timer', 'tcp-halfopen-timer',
                   'tcp-option', 'tcp-timewait-timer', 'tftp',
                   'timezone', 'tp-mc-skip-policy', 'traffic-priority',
                   'traffic-priority-level', 'two-factor-email-expiry', 'two-factor-fac-expiry',
                   'two-factor-ftk-expiry', 'two-factor-ftm-expiry', 'two-factor-sms-expiry',
                   'udp-idle-timer', 'user-server-cert', 'vdom-admin',
                   'vip-arp-range', 'virtual-server-count', 'virtual-server-hardware-acceleration',
                   'wad-affinity', 'wad-csvc-cs-count', 'wad-csvc-db-count',
                   'wad-source-affinity', 'wad-worker-count', 'wifi-ca-certificate',
                   'wifi-certificate', 'wimax-4g-usb', 'wireless-controller',
                   'wireless-controller-port']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = [[u'admin-https-ssl-versions']]

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def system_global(data, fos):
    vdom = data['vdom']
    system_global_data = data['system_global']
    system_global_data = flatten_multilists_attributes(system_global_data)
    filtered_data = filter_system_global_data(system_global_data)

    return fos.set('system',
                   'global',
                   data=filtered_data,
                   vdom=vdom)


def fortios_system(data, fos):
    login(data, fos)

    if data['system_global']:
        resp = system_global(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "system_global": {
            "required": False, "type": "dict",
            "options": {
                "admin-concurrent": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "admin-console-timeout": {"required": False, "type": "int"},
                "admin-https-pki-required": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "admin-https-ssl-versions": {"required": False, "type": "list",
                                             "choices": ["tlsv1-0", "tlsv1-1", "tlsv1-2"]},
                "admin-lockout-duration": {"required": False, "type": "int"},
                "admin-lockout-threshold": {"required": False, "type": "int"},
                "admin-login-max": {"required": False, "type": "int"},
                "admin-maintainer": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "admin-port": {"required": False, "type": "int"},
                "admin-restrict-local": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "admin-scp": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "admin-server-cert": {"required": False, "type": "str"},
                "admin-sport": {"required": False, "type": "int"},
                "admin-ssh-grace-time": {"required": False, "type": "int"},
                "admin-ssh-password": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "admin-ssh-port": {"required": False, "type": "int"},
                "admin-ssh-v1": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "admin-telnet-port": {"required": False, "type": "int"},
                "admintimeout": {"required": False, "type": "int"},
                "alias": {"required": False, "type": "str"},
                "allow-traffic-redirect": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "anti-replay": {"required": False, "type": "str",
                                "choices": ["disable", "loose", "strict"]},
                "arp-max-entry": {"required": False, "type": "int"},
                "asymroute": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "auth-cert": {"required": False, "type": "str"},
                "auth-http-port": {"required": False, "type": "int"},
                "auth-https-port": {"required": False, "type": "int"},
                "auth-keepalive": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "auth-session-limit": {"required": False, "type": "str",
                                       "choices": ["block-new", "logout-inactive"]},
                "auto-auth-extension-device": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "av-affinity": {"required": False, "type": "str"},
                "av-failopen": {"required": False, "type": "str",
                                "choices": ["pass", "off", "one-shot"]},
                "av-failopen-session": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "batch-cmdb": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "block-session-timer": {"required": False, "type": "int"},
                "br-fdb-max-entry": {"required": False, "type": "int"},
                "cert-chain-max": {"required": False, "type": "int"},
                "cfg-revert-timeout": {"required": False, "type": "int"},
                "cfg-save": {"required": False, "type": "str",
                             "choices": ["automatic", "manual", "revert"]},
                "check-protocol-header": {"required": False, "type": "str",
                                          "choices": ["loose", "strict"]},
                "check-reset-range": {"required": False, "type": "str",
                                      "choices": ["strict", "disable"]},
                "cli-audit-log": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "clt-cert-req": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "compliance-check": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "compliance-check-time": {"required": False, "type": "str"},
                "cpu-use-threshold": {"required": False, "type": "int"},
                "csr-ca-attribute": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "daily-restart": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "device-identification-active-scan-delay": {"required": False, "type": "int"},
                "device-idle-timeout": {"required": False, "type": "int"},
                "dh-params": {"required": False, "type": "str",
                              "choices": ["1024", "1536", "2048",
                                          "3072", "4096", "6144",
                                          "8192"]},
                "dst": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "endpoint-control-fds-access": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "endpoint-control-portal-port": {"required": False, "type": "int"},
                "failtime": {"required": False, "type": "int"},
                "fds-statistics": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "fds-statistics-period": {"required": False, "type": "int"},
                "fgd-alert-subscription": {"required": False, "type": "str",
                                           "choices": ["advisory", "latest-threat", "latest-virus",
                                                       "latest-attack", "new-antivirus-db", "new-attack-db"]},
                "fortiextender": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "fortiextender-data-port": {"required": False, "type": "int"},
                "fortiextender-vlan-mode": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "fortiservice-port": {"required": False, "type": "int"},
                "gui-certificates": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "gui-custom-language": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-date-format": {"required": False, "type": "str",
                                    "choices": ["yyyy/MM/dd", "dd/MM/yyyy", "MM/dd/yyyy",
                                                "yyyy-MM-dd", "dd-MM-yyyy", "MM-dd-yyyy"]},
                "gui-device-latitude": {"required": False, "type": "str"},
                "gui-device-longitude": {"required": False, "type": "str"},
                "gui-display-hostname": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "gui-ipv6": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "gui-lines-per-page": {"required": False, "type": "int"},
                "gui-theme": {"required": False, "type": "str",
                              "choices": ["green", "red", "blue",
                                          "melongene", "mariner"]},
                "gui-wireless-opensecurity": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "honor-df": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "hostname": {"required": False, "type": "str"},
                "igmp-state-limit": {"required": False, "type": "int"},
                "interval": {"required": False, "type": "int"},
                "ip-src-port-range": {"required": False, "type": "str"},
                "ips-affinity": {"required": False, "type": "str"},
                "ipsec-asic-offload": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "ipsec-hmac-offload": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "ipsec-soft-dec-async": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "ipv6-accept-dad": {"required": False, "type": "int"},
                "ipv6-allow-anycast-probe": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "language": {"required": False, "type": "str",
                             "choices": ["english", "french", "spanish",
                                         "portuguese", "japanese", "trach",
                                         "simch", "korean"]},
                "ldapconntimeout": {"required": False, "type": "int"},
                "lldp-transmission": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "log-ssl-connection": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "log-uuid": {"required": False, "type": "str",
                             "choices": ["disable", "policy-only", "extended"]},
                "login-timestamp": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "long-vdom-name": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "management-vdom": {"required": False, "type": "str"},
                "max-dlpstat-memory": {"required": False, "type": "int"},
                "max-route-cache-size": {"required": False, "type": "int"},
                "mc-ttl-notchange": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "memory-use-threshold-extreme": {"required": False, "type": "int"},
                "memory-use-threshold-green": {"required": False, "type": "int"},
                "memory-use-threshold-red": {"required": False, "type": "int"},
                "miglog-affinity": {"required": False, "type": "str"},
                "miglogd-children": {"required": False, "type": "int"},
                "multi-factor-authentication": {"required": False, "type": "str",
                                                "choices": ["optional", "mandatory"]},
                "multicast-forward": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "ndp-max-entry": {"required": False, "type": "int"},
                "per-user-bwl": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "policy-auth-concurrent": {"required": False, "type": "int"},
                "post-login-banner": {"required": False, "type": "str",
                                      "choices": ["disable", "enable"]},
                "pre-login-banner": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "private-data-encryption": {"required": False, "type": "str",
                                            "choices": ["disable", "enable"]},
                "proxy-auth-lifetime": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "proxy-auth-lifetime-timeout": {"required": False, "type": "int"},
                "proxy-auth-timeout": {"required": False, "type": "int"},
                "proxy-cipher-hardware-acceleration": {"required": False, "type": "str",
                                                       "choices": ["disable", "enable"]},
                "proxy-kxp-hardware-acceleration": {"required": False, "type": "str",
                                                    "choices": ["disable", "enable"]},
                "proxy-re-authentication-mode": {"required": False, "type": "str",
                                                 "choices": ["session", "traffic", "absolute"]},
                "proxy-worker-count": {"required": False, "type": "int"},
                "radius-port": {"required": False, "type": "int"},
                "reboot-upon-config-restore": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "refresh": {"required": False, "type": "int"},
                "remoteauthtimeout": {"required": False, "type": "int"},
                "reset-sessionless-tcp": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "restart-time": {"required": False, "type": "str"},
                "revision-backup-on-logout": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "revision-image-auto-backup": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "scanunit-count": {"required": False, "type": "int"},
                "security-rating-result-submission": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                "security-rating-run-on-schedule": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                "send-pmtu-icmp": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "snat-route-change": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "special-file-23-support": {"required": False, "type": "str",
                                            "choices": ["disable", "enable"]},
                "ssh-cbc-cipher": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "ssh-hmac-md5": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "ssh-kex-sha1": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "ssl-min-proto-version": {"required": False, "type": "str",
                                          "choices": ["SSLv3", "TLSv1", "TLSv1-1",
                                                      "TLSv1-2"]},
                "ssl-static-key-ciphers": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "sslvpn-cipher-hardware-acceleration": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                "sslvpn-kxp-hardware-acceleration": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                "sslvpn-max-worker-count": {"required": False, "type": "int"},
                "sslvpn-plugin-version-check": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "strict-dirty-session-check": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "strong-crypto": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "switch-controller": {"required": False, "type": "str",
                                      "choices": ["disable", "enable"]},
                "switch-controller-reserved-network": {"required": False, "type": "str"},
                "sys-perf-log-interval": {"required": False, "type": "int"},
                "tcp-halfclose-timer": {"required": False, "type": "int"},
                "tcp-halfopen-timer": {"required": False, "type": "int"},
                "tcp-option": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "tcp-timewait-timer": {"required": False, "type": "int"},
                "tftp": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "timezone": {"required": False, "type": "str",
                             "choices": ["01", "02", "03",
                                         "04", "05", "81",
                                         "06", "07", "08",
                                         "09", "10", "11",
                                         "12", "13", "74",
                                         "14", "77", "15",
                                         "87", "16", "17",
                                         "18", "19", "20",
                                         "75", "21", "22",
                                         "23", "24", "80",
                                         "79", "25", "26",
                                         "27", "28", "78",
                                         "29", "30", "31",
                                         "32", "33", "34",
                                         "35", "36", "37",
                                         "38", "83", "84",
                                         "40", "85", "41",
                                         "42", "43", "39",
                                         "44", "46", "47",
                                         "51", "48", "45",
                                         "49", "50", "52",
                                         "53", "54", "55",
                                         "56", "57", "58",
                                         "59", "60", "62",
                                         "63", "61", "64",
                                         "65", "66", "67",
                                         "68", "69", "70",
                                         "71", "72", "00",
                                         "82", "73", "86",
                                         "76"]},
                "tp-mc-skip-policy": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "traffic-priority": {"required": False, "type": "str",
                                     "choices": ["tos", "dscp"]},
                "traffic-priority-level": {"required": False, "type": "str",
                                           "choices": ["low", "medium", "high"]},
                "two-factor-email-expiry": {"required": False, "type": "int"},
                "two-factor-fac-expiry": {"required": False, "type": "int"},
                "two-factor-ftk-expiry": {"required": False, "type": "int"},
                "two-factor-ftm-expiry": {"required": False, "type": "int"},
                "two-factor-sms-expiry": {"required": False, "type": "int"},
                "udp-idle-timer": {"required": False, "type": "int"},
                "user-server-cert": {"required": False, "type": "str"},
                "vdom-admin": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "vip-arp-range": {"required": False, "type": "str",
                                  "choices": ["unlimited", "restricted"]},
                "virtual-server-count": {"required": False, "type": "int"},
                "virtual-server-hardware-acceleration": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                "wad-affinity": {"required": False, "type": "str"},
                "wad-csvc-cs-count": {"required": False, "type": "int"},
                "wad-csvc-db-count": {"required": False, "type": "int"},
                "wad-source-affinity": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                "wad-worker-count": {"required": False, "type": "int"},
                "wifi-ca-certificate": {"required": False, "type": "str"},
                "wifi-certificate": {"required": False, "type": "str"},
                "wimax-4g-usb": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "wireless-controller": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "wireless-controller-port": {"required": False, "type": "int"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_system(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, Bojan Vitnik <bvitnik@mainstream.rs>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: xenserver_guest_facts
short_description: Gathers facts for virtual machines running on Citrix XenServer host or pool
description: >
   This module can be used to gather essential VM facts.
version_added: '2.8'
author:
- Bojan Vitnik (@bvitnik) <bvitnik@mainstream.rs>
notes:
- Minimal supported version of XenServer is 5.6.
- Module was tested with XenServer 6.5, 7.1 and 7.2.
- 'XenAPI Python library can be acquired from XenServer SDK (downloadable from Citrix website) or by running C(pip install XenAPI) (possibly very old
   version, not compatible with Python 3.x). Latest version can also be acquired from GitHub:
   https://raw.githubusercontent.com/xapi-project/xen-api/master/scripts/examples/python/XenAPI.py'
- 'If no scheme is specified in C(hostname), module defaults to C(http://) because C(https://) is problematic in most setups. Make sure you are
   accessing XenServer host in trusted environment or use C(https://) scheme explicitly.'
- 'To use C(https://) scheme for C(hostname) you have to either import host certificate to your OS certificate store or use C(validate_certs: no)
   which requires XenAPI library from XenServer 7.2 SDK or newer and Python 2.7.9 or newer.'
requirements:
- python >= 2.6
- XenAPI
options:
  name:
    description:
    - Name of the VM to gather facts from.
    - VMs running on XenServer do not necessarily have unique names. The module will fail if multiple VMs with same name are found.
    - In case of multiple VMs with same name, use C(uuid) to uniquely specify VM to manage.
    - This parameter is case sensitive.
    type: str
    required: yes
    aliases: [ name_label ]
  uuid:
    description:
    - UUID of the VM to gather fact of. This is XenServer's unique identifier.
    - It is required if name is not unique.
    type: str
extends_documentation_fragment: xenserver.documentation
'''

EXAMPLES = r'''
- name: Gather facts
  xenserver_guest_facts:
    hostname: "{{ xenserver_hostname }}"
    username: "{{ xenserver_username }}"
    password: "{{ xenserver_password }}"
    name: testvm_11
  delegate_to: localhost
  register: facts
'''

RETURN = r'''
instance:
    description: Metadata about the VM
    returned: always
    type: dict
    sample: {
        "cdrom": {
            "type": "none"
        },
        "customization_agent": "native",
        "disks": [
            {
                "name": "testvm_11-0",
                "name_desc": "",
                "os_device": "xvda",
                "size": 42949672960,
                "sr": "Local storage",
                "sr_uuid": "0af1245e-bdb0-ba33-1446-57a962ec4075",
                "vbd_userdevice": "0"
            },
            {
                "name": "testvm_11-1",
                "name_desc": "",
                "os_device": "xvdb",
                "size": 42949672960,
                "sr": "Local storage",
                "sr_uuid": "0af1245e-bdb0-ba33-1446-57a962ec4075",
                "vbd_userdevice": "1"
            }
        ],
        "domid": "56",
        "folder": "",
        "hardware": {
            "memory_mb": 8192,
            "num_cpu_cores_per_socket": 2,
            "num_cpus": 4
        },
        "home_server": "",
        "is_template": false,
        "name": "testvm_11",
        "name_desc": "",
        "networks": [
            {
                "gateway": "192.168.0.254",
                "gateway6": "fc00::fffe",
                "ip": "192.168.0.200",
                "ip6": [
                    "fe80:0000:0000:0000:e9cb:625a:32c5:c291",
                    "fc00:0000:0000:0000:0000:0000:0000:0001"
                ],
                "mac": "ba:91:3a:48:20:76",
                "mtu": "1500",
                "name": "Pool-wide network associated with eth1",
                "netmask": "255.255.255.128",
                "prefix": "25",
                "prefix6": "64",
                "vif_device": "0"
            }
        ],
        "other_config": {
            "base_template_name": "Windows Server 2016 (64-bit)",
            "import_task": "OpaqueRef:e43eb71c-45d6-5351-09ff-96e4fb7d0fa5",
            "install-methods": "cdrom",
            "instant": "true",
            "mac_seed": "f83e8d8a-cfdc-b105-b054-ef5cb416b77e"
        },
        "platform": {
            "acpi": "1",
            "apic": "true",
            "cores-per-socket": "2",
            "device_id": "0002",
            "hpet": "true",
            "nx": "true",
            "pae": "true",
            "timeoffset": "-25200",
            "vga": "std",
            "videoram": "8",
            "viridian": "true",
            "viridian_reference_tsc": "true",
            "viridian_time_ref_count": "true"
        },
        "state": "poweredon",
        "uuid": "e3c0b2d5-5f05-424e-479c-d3df8b3e7cda",
        "xenstore_data": {
            "vm-data": ""
        }
    }
'''

HAS_XENAPI = False
try:
    import XenAPI
    HAS_XENAPI = True
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.xenserver import (xenserver_common_argument_spec, XAPI, XenServerObject, get_object_ref,
                                            gather_vm_params, gather_vm_facts)


class XenServerVM(XenServerObject):
    """Class for managing XenServer VM.

    Attributes:
        vm_ref (str): XAPI reference to VM.
        vm_params (dict): A dictionary with VM parameters as returned
            by gather_vm_params() function.
    """

    def __init__(self, module):
        """Inits XenServerVM using module parameters.

        Args:
            module: Reference to AnsibleModule object.
        """
        super(XenServerVM, self).__init__(module)

        self.vm_ref = get_object_ref(self.module, self.module.params['name'], self.module.params['uuid'], obj_type="VM", fail=True, msg_prefix="VM search: ")
        self.gather_params()

    def gather_params(self):
        """Gathers all VM parameters available in XAPI database."""
        self.vm_params = gather_vm_params(self.module, self.vm_ref)

    def gather_facts(self):
        """Gathers and returns VM facts."""
        return gather_vm_facts(self.module, self.vm_params)


def main():
    argument_spec = xenserver_common_argument_spec()
    argument_spec.update(
        name=dict(type='str', aliases=['name_label']),
        uuid=dict(type='str'),
    )

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True,
                           required_one_of=[
                               ['name', 'uuid'],
                           ],
                           )

    result = {'failed': False, 'changed': False}

    # Module will exit with an error message if no VM is found.
    vm = XenServerVM(module)

    # Gather facts.
    result['instance'] = vm.gather_facts()

    if result['failed']:
        module.fail_json(**result)
    else:
        module.exit_json(**result)


if __name__ == '__main__':
    main()
